package client;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.text.*;

/**
 * The displayable part of our instant messenger application.
 */
public class IMFrame extends JFrame implements ActionListener {
	private static final long serialVersionUID = 1L;

	/** How to send messages. */
	private MessageSender sender;

	/** Where the user list is displayed. */
	private JList userList = new JList();
	/** Where the user can enter a message to send. */
	private JTextField sendField = new JTextField(32);
	/** Where the incoming messages are displayed. */
	private JTextPane receiveArea = new JTextPane();
	private Document document = receiveArea.getDocument();
	private JScrollPane receiveScroller = new JScrollPane(receiveArea);

	/**
	 * Sets up the GUI with all the necessary elements.
	 */
	public IMFrame(MessageSender sender, String name) {
		/* frame stuff */
		super("Instant Messenger for " + name);

		addWindowListener(new WindowAdapter() { // To make sure, that everything shuts down
			public void windowClosing(WindowEvent e) {
				System.exit(0);
			}
		});
		setBackground(Color.white);
		setForeground(Color.white);

		/* the sender */
		this.sender = sender;

		/* all the swing elements */
		// SENDING
		sendField.setBorder(BorderFactory.createTitledBorder("Send Message"));
		sendField.addActionListener(this);
		sendField.addFocusListener(new FocusAdapter() {
			public void focusGained(FocusEvent e) {
				sendField.requestFocus();
			}
		});

		// RECEIVING
		receiveScroller.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		receiveScroller.setBorder(BorderFactory.createTitledBorder("Messages for You"));
		receiveArea.setEditable(false);
		initStyles();

		// USERS
		JScrollPane userScroller = new JScrollPane(userList);
		userScroller.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		userScroller.setBorder(BorderFactory.createTitledBorder("Users"));
		userList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

		// ALL TOGETHER
		JPanel messagePanel = new JPanel(new BorderLayout());
		messagePanel.add(sendField, BorderLayout.NORTH);
		messagePanel.add(receiveScroller, BorderLayout.CENTER);
		JSplitPane splitter = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, userScroller, messagePanel);
		splitter.setDividerLocation(0.4);

		getContentPane().add(splitter);
	}

	/** Sets the styles (= colors) for the text pane. */
	private void initStyles() {
		// the default of everything
		Style def = StyleContext.getDefaultStyleContext().getStyle(StyleContext.DEFAULT_STYLE);

		Style black = receiveArea.addStyle("black", def);
		StyleConstants.setForeground(black, Color.black);

		Style blue = receiveArea.addStyle("blue", def);
		StyleConstants.setForeground(blue, Color.blue);

		Style red = receiveArea.addStyle("red", def);
		StyleConstants.setForeground(red, Color.red);
	}

	/**
	 * When a message arrives, this will display it on the screen.
	 */
	void displayUserMessage(String msg) {
		try {
			document.insertString(0, msg + "\n", receiveArea.getStyle("blue"));
		} catch (BadLocationException ble) {
		}
		receiveArea.setCaretPosition(0);
	}

	/**
	 * Updates the user list.
	 */
	void setUserList(User[] users) {
		Object oldSelection = userList.getSelectedValue();
		userList.setListData(users);
		if (oldSelection != null) {
			userList.setSelectedValue(oldSelection, false);
		}
	}

	/**
	 * This will send a message to another user.
	 */
	public void actionPerformed(ActionEvent e) {
		User user = (User)userList.getSelectedValue();
		if (user != null) {
			try {
				if (sender.sendMessage(sendField.getText(), user)) {
					document.insertString(0, "Sent to " + user.getName() + " -> " + sendField.getText() + "\n", receiveArea.getStyle("black"));
				} else {
					document.insertString(0, "Failed to send to " + user.getName() + "\n", receiveArea.getStyle("red"));
				}
			} catch (BadLocationException ble) {
			}
			receiveArea.setCaretPosition(0);
			sendField.setText("");
		}
	}
}